// M5Stamp PICO TCP/IP Termianl Server 1
// １対n接続 Terminal

#include <Arduino.h>
#include <WiFi.h>
#include <WiFiClient.h>
#include <WiFiAP.h>

#define OLDMONITER

#ifdef OLDMONITER
#include "OLED.h"
// OLED用
char str[80] ;
OLED oled ;
#endif

// Server側:SSIDとパスワード
#define SERVERPORT 23
const char* ssid     = "STAMP-PICO";
const char* password = "STAMP-PICO";

WiFiServer server(SERVERPORT,4);  // Terminal Port：最大接続数４

#define ListenMax 4
#define BuffMaxLen 80
WiFiClient client[ListenMax] ;
int readBuffLen[ListenMax] ;
char readBuff[ListenMax][BuffMaxLen] ;

#define LED1 26
#define LED2 18
int led1Stat ;
int led2Stat ;
bool flagChangeLedStat ;

// コマンド解析
void comanExec(char *commanLine) {
  char command[BuffMaxLen] ;
  int commandNo = 0 ;
  int commandNo2 = 0 ;
  int commandLen = 0 ;
  int readLen = 0 ;
  memset(command,0,BuffMaxLen) ;
  while(commanLine[readLen] != '\0') {
    if (commanLine[readLen] == ',') {
      readLen ++ ;
      commandLen = 0 ;
      memset(command,0,BuffMaxLen) ;
      continue ;
    }
    if (commanLine[readLen] <= ' ') {
      readLen ++ ;
      continue ;
    }
    if (commanLine[readLen] == ':') {
      readLen ++ ;
      commandLen = 0 ;
      break ;
    }
    command[commandLen] = commanLine[readLen] ;
    commandLen ++ ; readLen ++ ;
  }

#ifdef OLDMONITER
  sprintf(str,"CMD:%s",command) ; oled.Print(str) ;
#else
  Serial.print("Command1:") ;
  Serial.println(command) ;
#endif

  if (strcmp(command,"LED1") == 0) { commandNo = 1; }
  else if (strcmp(command,"LED2") == 0) { commandNo = 2; }

  commandLen = 0 ;
  memset(command,0,BuffMaxLen) ;
  while(commanLine[readLen] != '\0') {
    if (commanLine[readLen] == ',') {
      readLen ++ ;
      commandLen = 0 ;
      memset(command,0,BuffMaxLen) ;
      continue ;
    }
    if (commanLine[readLen] <= ' ') {
      readLen ++ ;
      continue ;
    }
    if (commanLine[readLen] == ':') {
      readLen ++ ;
      commandLen = 0 ;
      break ;
    }
    command[commandLen] = commanLine[readLen] ;
    commandLen ++ ; readLen ++ ;
  }

  if (strcmp(command,"ON") == 0) { commandNo2 = 1; }
  else if (strcmp(command,"OFF") == 0) { commandNo2 = 2; }

#ifdef OLDMONITER
  sprintf(str,":%s",command) ; oled.Println(str) ;
#else
  Serial.print("Command2:") ;
  Serial.println(command) ;
  Serial.print("commandNo2:") ;
  Serial.println(commandNo2) ;
#endif

  for (int i=0;i<BuffMaxLen;i++) {
    if (command[i] == '\0') break ;
  }
  switch (commandNo) {
  case 1: 
    flagChangeLedStat = true ;
    if (strcmp(command,"ON") == 0) {
      led1Stat = HIGH ;
    } else {
      led1Stat = LOW ;
    }
    digitalWrite(LED1,led1Stat) ;
    break ;
  case 2: 
    flagChangeLedStat = true ;
    if (strcmp(command,"ON") == 0) {
      led2Stat = HIGH ;
    } else {
      led2Stat = LOW ;
    }
    digitalWrite(LED2,led2Stat) ;
    break ;
  }
  
}

// セットアップ
void setup() {
  // ---- LED設定
  led1Stat = LOW ;
  led2Stat = LOW ;
  flagChangeLedStat = false ;
  pinMode(LED1,OUTPUT) ;
  pinMode(LED2,OUTPUT) ;
  digitalWrite(LED1,led1Stat) ;
  digitalWrite(LED2,led2Stat) ;

  // ----- 受信バッファ初期化
  for (int i=0;i<ListenMax;i++) {
    readBuffLen[i] = 0 ;
    readBuff[i][0] = '\0' ;
  }

  // ----- Wi-Fi AP設定
  WiFi.softAP(ssid, password);  // Server APを設定
  IPAddress Ip(192, 168, 10, 10);     // Server IP を固定
  IPAddress NMask(255, 255, 255, 0);  // Server SubNetMask
  WiFi.softAPConfig(Ip, Ip, NMask);   // IP , Gateway , SubNetMask
  IPAddress myIP = WiFi.softAPIP();   // APを開設
  server.begin(); // Server Soket Listen開始

#ifdef OLDMONITER
  // ---- OLED
  oled.init();
  sprintf(str,"WIFI ACCESS POINT") ; oled.Println(str) ;
  sprintf(str,"Please connect:%s",ssid) ; oled.Println(str) ;
  sprintf(str,"Then access to:%s",myIP.toString()) ; oled.Println(str) ;
#else
  Serial.begin(9600);
  Serial.println("WIFI ACCESS POINT");  //print string.
  Serial.printf("Please connect:%s \nThen access to:",ssid);
  Serial.println(myIP);
#endif
}

// 処理ループ
void loop() {
  bool flagSendLedStat = flagChangeLedStat ;
  flagChangeLedStat = false ;
  for (int i=0;i<ListenMax;i++) {
    if (client[i].connected()) {
      // 接続済チャンネルの処理
      if (client[i].available()) {  // if there's bytes to read from the client.
        char c = client[i].read();  // store the read a byte.
        // client[i].print(c) ;  // ECHO BACK
        if (c == '\n') {
          // コマンド受信
          comanExec(readBuff[i]) ;

          readBuffLen[i] = 0 ;
          readBuff[i][readBuffLen[i]] = '\0' ;
        } else {
          readBuff[i][readBuffLen[i]] = c ;
          if (readBuffLen[i] < BuffMaxLen - 1) {
            readBuffLen[i] ++ ;
          }
          readBuff[i][readBuffLen[i]] = '\0' ;
        }
      }
      if (flagSendLedStat) {
        client[i].print("LED1-STAT:") ;
        if (led1Stat == HIGH) {
          client[i].println("ON") ;
        } else {
          client[i].println("OFF") ;
        }
        client[i].print("LED2-STAT:") ;
        if (led2Stat == HIGH) {
          client[i].println("ON") ;
        } else {
          client[i].println("OFF") ;
        }
      }
    } else {
      // 未接続チャンネルの処理：接続待ち
      client[i] = server.available();       // listen for incoming clients.
      if (client[i]) {                      // if you get a client.
        while (!client[i].connected());
        client[i].println("Hello!") ;

#ifdef OLDMONITER
        sprintf(str,"New Client:%s",client[i].remoteIP().toString()) ; oled.Println(str) ;
#else
        Serial.print("New Client:");
        Serial.println(client[i].remoteIP());
#endif
      }
    }
  }
}
